program E_calc
USE omp_lib
! Program calculating the electric field at a set of 
! observation points based on the surface current 
! density, J, fetched from the file, 'J_data'.
USE disc_mod	
USE calculation_mod_omp
implicit none

! Declaring program parameters
integer, parameter :: out_unit=20, in_unit=2

! Declaring program variables
integer                         :: i, M, nx, ny, nz, ne, n, j, &
                                   l, nc, q, p, quad_order, &
                                   disc_number, counter, p_t, &
                                   ax1, ax2, circ_ax, tot_field
real                            :: y_min, y_max, x_min, x_max, &
                                   z_min, z_max, omega, my_0, &
                                   area_tot, temp_r, E_inc_0, &
                                   E_inc_hat(3), lambda, & 
                                   k_vec(3), k_hat(3), k, de, &
                                   circ_r
complex                         :: integral, sum_p
real, dimension(:), allocatable :: Area_list
type(Vector_c) , dimension(3)   :: G_tensor
type(Point), dimension(:), allocatable     :: r_list
type(Structure)                            :: struct 
type(Point) ,dimension(:), allocatable     :: point_list 
type(Vector_c) , dimension(:), allocatable :: J_list, &
                                              J_list_quad, & 
                                              E_res, E_inc
character (len=40)                         :: file_J                                              
                                                          

file_J='J_data'

! Reading input parameteres from J_data file
open(unit=in_unit, file=trim(file_J))
read(in_unit, *) n, ne, lambda, quad_order, disc_number, E_inc_0
read(in_unit, *) E_inc_hat, k_hat
close(in_unit)
 
!----------------------------------------------
!--------Input Parameters-E_calc-(begin)-------
!----------------------------------------------

! Choose type of observation point list 
p_t=1 !(1: xy, 2:yz, 3:xz, 4: Circle, 5:Point)

x_min=-40.0
x_max=40.0
y_min=-40.0
y_max=40.0
z_min=0.0
z_max=0.0

! Choose Resolution
de=0.1*lambda 

! Choose total or scattered field 
tot_field=1.0 !(1 for total, 0 for scattered)

! if p_t =4,5
! Choose circle radius and plane
circ_r=40
circ_ax=2 !(1:xy, 2:xz, 3:yz)

! if p_t=5

!---------------------------------------
!--------Input Parameters-(end)-------
!---------------------------------------

!uncomment if parameters are included from input file
!include 'Run_circ03_E.par'



my_0=4*pi*1E-7
k=2.0*pi/lambda
omega=k*c
k_vec=k*k_hat

! Choosing discretization algorithm which fills struct
if (disc_number.eq.1)then
  call stripack_convert(struct, n)
else if (disc_number.eq.2)then
  call recursive_triangulation(struct, n)
end if 
ne=size(struct%elements)	

! Calculates area of the elements and fills Area_list with 
! the area of the corresponding element
allocate(Area_list(ne))
area_tot=0.0
do i=1,ne
  nc=struct%elements(i)%nr_of_corners
  allocate(point_list(nc))
  do j=1,nc
    point_list(j)=struct%points(struct%elements(i)%corners(j))
  end do
  temp_r=area(point_list)
  area_tot=area_tot+temp_r
  Area_list(i)=temp_r
  deallocate(point_list)
end do

! Reading current density data from J_data file
open(unit=in_unit, file=trim(file_J))
read(in_unit, *) !n, ne, lambda
read(in_unit, *)
allocate(J_list(quad_order*ne))
do i=1,quad_order*ne
  read(in_unit, *), J_list(i)%vector
end do
close(in_unit)

print *, 'Done reading J_data!'

! Creating list of M observation points

nx=ceiling(1+(x_max-x_min)/de)
ny=ceiling(1+(y_max-y_min)/de)
nz=ceiling(1+(z_max-z_min)/de)

if (p_t < 4)then
  print *,'nx,ny,nz:', nx,ny,nz
	M=nx*ny*nz
	allocate(r_list(M))
end if

if (p_t .eq. 1)then ! xy-plot
  ax1=1
  ax2=2
  do i=1,ny
    do j=1,nx
      r_list((i-1)*nx+j)%point= &
      (/ x_min+de*(j-1), y_min+de*(i-1), z_min /)
    end do
  end do
else if (p_t .eq. 2)then ! yz-plot
  ax1=2
  ax2=3
  do i=1,nz
    do j=1,ny
      r_list((i-1)*ny+j)%point=(/ x_min, y_min+de*(j-1), & 
      z_min+de*(i-1) /)
    end do
  end do
else if (p_t .eq. 3)then ! xz-plot
  ax1=1
  ax2=3
  do i=1,nz
    do j=1,nx
      r_list((i-1)*nx+j)%point=(/ x_min+de*(j-1), y_min, & 
      z_min+de*(i-1) /)
    end do
  end do
else if (p_t .eq. 4)then ! Circle
  ! Creates a circle in the plane set in the input file
  M=ceiling(2.0*pi*circ_r*200/lambda)
  allocate(r_list(M))
  if (circ_ax .eq. 1)then !xy
    ax1=1
    ax2=2
      do i=1, M
        r_list(i)%point=(/ circ_r*cos(2.0*pi*(i-1)/(M-1)), &
        circ_r*sin(2.0*pi*(i-1)/(M-1)) , 0.0/)   
    end do
  else if (circ_ax .eq. 2)then !xz
    ax1=1
    ax2=3
      do i=1, M
        r_list(i)%point=(/ circ_r*cos(2.0*pi*(i-1)/(M-1)), 0.0, &
        circ_r*sin(2.0*pi*(i-1)/(M-1))/)
      end do
  else if (circ_ax .eq. 3)then !yz
    ax1=2
    ax2=3
    do i=1, M
      r_list(i)%point=(/ 0.0, circ_r*cos(2.0*pi*(i-1)/(M-1)), &
      circ_r*sin(2.0*pi*(i-1)/(M-1)) /)
    end do
  end if
else if (p_t .eq. 5)then ! Single Point
  M=1
  allocate(r_list(M))
  ax1=1
  ax2=2
  r_list(1)%point=(/ x_min, y_min, z_min /)
end if

print *, 'Total number of observation points:', M
print *, 'Calculating E-field amplitude...'

! Creating list of points on a straight line
!M=2021
!allocate(r_list(M))
!do i=1,(M)
!  r_list(i)%point=(/ 5.0, 0.0, -0.2+(i-1)*0.01/)
!r_list(i+M/2)%point=(/ -1.99-(i-1)*0.01, 0.0, 0.0/)
!end do

! Calculating incoming electric field E_inc at 
! all observation points in r_list
allocate(E_inc(M))
do i=1,(M)
  temp_r=dot_prod_r(k_vec,r_list(i)%point)
  E_inc(i)%vector=E_inc_0*E_inc_hat*exp(im*temp_r)
end do



! Calculating Resulting Electric field, E_res, 
! in all observation points in r_list
! Quad order = 3
if (quad_order.eq.3)then
allocate(E_res(M))
counter=0

!$omp parallel private ( i,l, j,p,q, G_tensor, sum_p, integral) &
! !$omp reduction ( + : E_res(1)%vector(1)) ) &
!$omp shared ( Area_list, omega, my_0, r_list, ne, k, J_list) &
!$omp shared ( M, quad_order, E_res, counter, struct )

!$omp do
do i=1,M ! for all r
  ! M must here be more than 100. Else: comment out
  if (mod(counter,M/100).eq. 0 .and. counter .ne. 0)then 
    write (*, "(i5,a1)")int(ceiling(counter*100.0/M)),'%'
  end if
  if (r_list(i)%point(1)**2+r_list(i)%point(2)**2 &
      +r_list(i)%point(3)**2>1.00)then	
    do l=1,3 !for each of the three directions
      integral=(0.0,0.0) 	
      do j=1,ne !for every element in the discretization
        sum_p=(0.0,0.0)
        do p=1,3 ! for each of the three evaluation points
          do q=1,3
            G_tensor(l)%vector(q)= &
            G(r_list(i)%point,struct%quadpoints(j,p)%point,l,q,k)
          end do				
          sum_p=sum_p+dot_prod_c(G_tensor(l)%vector, &
          J_list((j-1)*3+p)%vector)
        end do
        integral=integral+(1.0/3.0)*sum_p*Area_list(j)
      end do
      E_res(i)%vector(l)=(tot_field*E_inc(i)%vector(l) &
      -(omega*my_0/im)*integral)/E_inc_0
    end do
  else
    E_res(i)%vector=(/ (0.0, 0.0), (0.0,0.0), (0.0,0.0)/)
  end if
  counter=counter+1
end do
!$omp end do
!$omp end parallel
end if
if (quad_order.eq.1)then	
! Calculating Resulting Electric field, E_res, in all 
! observation points in r_list having when quad order = 1
allocate(E_res(M))
!allocate(point_list(3))
do i=1,M ! for alle r
  ! M must here be more than 100. Else: comment out
  if (mod(i,M/100).eq. 0)then
    write (*, "(i5,a1)")int(ceiling(i*100.0/M)),'%' 
  end if
  if (r_list(i)%point(1)**2+r_list(i)%point(2)**2 &
  +r_list(i)%point(3)**2>1.00)then	
    do l=1,3 !for each of the three directions
      integral=(0.0,0.0) 	
      do j=1,ne	!for every element in the discretization
        do q=1,3
          G_tensor(l)%vector(q)=G(r_list(i) &
          %point,struct%midpoints(j)%point,l,q,k)
        end do				
        integral=integral+dot_prod_c(G_tensor(l)%vector, &
        J_list(j)%vector)*Area_list(j)
      end do
      E_res(i)%vector(l)=tot_field*E_inc(i)%vector(l) &
      -(omega*my_0/im)*integral
    end do
    else
      E_res(i)%vector=(/ (0.0, 0.0), (0.0,0.0), (0.0,0.0)/)
    end if
end do	
end if
print *, 'Done calculating electric field amplitude!'

! Printing the resulting electric field 
! amplitude, |E|/|E_0|, to file
open (unit=out_unit,file="results_E.dat",action="write", &
status="replace")
write (out_unit, "(a, i3,a,i8)") 'Output type:', p_t, ', ne: ', ne
write (out_unit, "(I7, f8.3, f9.2)") M, de, lambda
do i=1,M
  !write (*, "(f8.2,f8.2,f9.4)") r_list(i)%point(ax1), &
  !r_list(i)%point(ax2), sqrt(abs(E_res(i)%vector(1))**2 &
  !+abs(E_res(i)%vector(2))**2+abs(E_res(i)%vector(3))**2)
  write (out_unit, "(f8.2,f8.2,e14.7)") r_list(i)%point(ax1),&
  r_list(i)%point(ax2), sqrt(abs(E_res(i)%vector(1))**2 &
  +abs(E_res(i)%vector(2))**2+abs(E_res(i)%vector(3))**2)
end do
close (out_unit)


end program



