program J_calc
! Part 1 of the main program calculating the current density J
USE disc_mod
USE calculation_mod_omp
implicit none

! Declaring program variables 

! Variables for the output data
integer, parameter :: out_unit=20, in_unit=2

! Variables for the discretization
integer :: n, ne, i, nc, j, l, q, np, nx,ny,nz
type(Point) ,dimension(:), allocatable :: point_list  
type(Structure) :: struct 
real area_tot

! Variables for the calculation
type(Point), dimension(:), allocatable     :: r_list
type(Vector_c) , dimension(:), allocatable :: H_inc, J_list, &
                                              E_res, E_inc
type(Vector_c) , dimension(3)    :: G_tensor
complex                          :: temp_c, integral
complex, dimension(3)            :: temp_c_vec
complex, dimension(3,3)          :: Greens
real                             :: k, lambda, omega, H_inc_0, &
                                    E_inc_0, temp_r
real , dimension(3)              :: k_vec, H_inc_hat, E_inc_hat, & 
                                    k_hat, temp_r_vec
real, dimension(:), allocatable  :: Area_list
integer                          :: M, temp_i, quad_order, & 
                                    disc_number
! Constants: 
real                             :: my_0, eps_0, y_min, y_max, &
                                    z_min, z_max, x_min, x_max, de 
logical                          :: KA

!---------------------------------------
!--------Input Parameters-(begin)-------
!---------------------------------------

!print *, 'Choose number of discretization points:'
n=258 !(rec:6, 18, 66, 258, 1026, 4098, 16386, 65538, 262146, ...)

!print *,'Choose wavelength in units of sphere radius R:'
lambda=40.0

! Print Choose polarization of incoming EM wave 
E_inc_hat=(/ 0.0, 0.0, 1.0 /)

! Choose direction of incoming wave
k_hat=(/ 1.0, 0.0, 0.0/)

! Set quadrature order (1 or 3)
quad_order=3

! Use KA
KA = .false.

! Choose the discretization algorithm which fills struct with data
disc_number=2

!---------------------------------------
!--------Input Parameters-(end)-------
!---------------------------------------

!uncomment if parameters are instead included from input file
!include 'Run_circ03_J.par'

if (disc_number.eq.1)then
  call stripack_convert(struct, n)
else if (disc_number.eq.2)then
  call recursive_triangulation(struct, n)
end if 	

ne=size(struct%elements)

print *,' ' 
print *,'List of points:'
do i=1,5 !n
  write (*,"(I5,F10.5,F10.5,F10.5)"), i,struct%points(i)%point(1:3)
end do

print *, ' '
print *,'List of elements:'
do i=1,5 !ne
  write (*,"(10i5)"), i, struct%elements(i)%corners(1:struct &
  %elements(i)%nr_of_corners)
end do
print *, '              '

allocate(Area_list(ne))
! Calculates area of the elements and fills Area_list with 
! the area of the corresponding element
area_tot=0.0
!print *,'Areas:'
do i=1,ne
  nc=struct%elements(i)%nr_of_corners
  allocate(point_list(nc))
  do j=1,nc
    point_list(j)=struct%points(struct%elements(i)%corners(j))
  end do
  temp_r=area(point_list)
  area_tot=area_tot+temp_r
  Area_list(i)=temp_r
  deallocate(point_list)
end do

! Uncomment if area distribution shall be calculated
!call area_dist(struct, disc_number, Area_list)
!read(*,*)

! Prints neighbours:
print *, ' '
print *, 'Printing neighbour elements: '
print *, 'pair nr: ', '             ','elements in pair:', & 
'        ','common corners:'
do i=1, 5 !size(struct%neighbours)
  print *, i, struct%neighbours(i)%elements(1:2), &
  struct%neighbours(i)%corners(1:2)
end do

! Prints midpoints
print *, ' '
print *, 'Printing midpoints: '
print *, 'Element nr: ', '          ','Midpoint: '
do i=1,5 ! ne
  write (*,"(I5,F12.7,F12.7,F12.7)"), &
  i,struct%midpoints(i)%point(1:3)
end do 

! Outputs some information
print *, ' '
print *, 'Number of elements: ', ne
print *, 'Total relative area: ', area_tot/(16*atan(1.0))

! Scattering Calculation

! Defining the constants
my_0=4*pi*1E-7
eps_0=1.0/(my_0*c**2)

! Defining Variables 
E_inc_0=1.0	
k=2.0*pi/lambda
omega=k*c
k_vec=k*k_hat
H_inc_hat=cross_r(k_hat, E_inc_hat)
H_inc_0=E_inc_0/(my_0*c)

! Printing Variable information
write (*, "(a,i5)"),    ' disc_number:           ', disc_number
write (*, "(a,i8)"),    ' discretization points: ', n
write (*, "(a,l4)"),    ' KA: ', KA
write (*, "(a,e12.5)")  ' eps_0=', eps_0
write (*, "(a,e12.5)")  ' my_0=', my_0
write (*, "(a,f9.3)")   ' E_inc_0=', E_inc_0
write (*, "(a,3f9.3)")  ' E_inc_hat=', E_inc_hat
write (*, "(a,e12.5)")  ' H_inc_0=', H_inc_0
write (*, "(a,3f9.3)")  ' H_inc_hat=', H_inc_hat
write (*, "(a,e12.5)")  ' k = ', k
write (*, "(a,3f9.3)")  ' k_hat=', k_hat
write (*, "(a,f9.3, a)")' lambda = ', lambda, 'R'
write (*, "(a,e12.5)")  ' omega = ', omega
write (*,*) ' '

allocate(J_list(quad_order*ne))
if ( KA .eqv. .true. )then
  ! Calculating incoming H-field at scatterer surface (KA approx)
  ! using either 1-point or 3-point Gaussian quadrature method
  if (quad_order.eq.1)then
    allocate(H_inc(ne))
    do i=1, ne
      temp_r=dot_prod_r(k_vec,struct%midpoints(i)%point)
      H_inc(i)%vector=H_inc_0*H_inc_hat*cexp(im*temp_r)
    end do
  else if (quad_order.eq.3)then
    allocate(H_inc(3*ne))
    do i=1,ne
      do j=1,3
        temp_r=dot_prod_r(k_vec,struct%quadpoints(i,j)%point)
        H_inc((i-1)*3+j)%vector=H_inc_0*H_inc_hat*cexp(im*temp_r)
      end do
    end do
  end if	
  ! Calculating el current density J_list at surface (KA approx)
  do i=1, ne
    temp_c_vec=norm_vec_c(struct%points(struct%elements(i) &
    %corners(1))%point, &
    struct%points(struct%elements(i)%corners(2))%point, &
    struct%points(struct%elements(i)%corners(3))%point, &
    struct%midpoints(i)%point )
    do j=1,quad_order
      J_list((i-1)*quad_order+j)%vector &
      =2.0*cross_c(temp_c_vec, H_inc((i-1)*quad_order+j)%vector)
    end do
  end do
else
  ! Calculating el curr density J_list at surface (linear system)
  if (quad_order.eq.1)then
    print *, 'Generating current densities by ' 
    print *, '1st order gauss quadrature integrals...'
    J_list=J_gen(k, k_hat, struct, Area_list, my_0, &
           E_inc_hat*E_inc_0)
  else if (quad_order.eq.3)then
    print *, 'Generating current densities by '
    print *, '3rd order gauss quadrature integrals...'
    J_list=J_gen_quad(k, k_hat, struct, Area_list, my_0, &
           E_inc_hat*E_inc_0)
  else
    print *,'Wrong quadrature order!'
  end if
end if

open (unit=out_unit,file='J_data', action="write",status="replace")
  write (out_unit, *) n, ne, lambda, quad_order, &
  disc_number, E_inc_0
  write (out_unit, *) E_inc_hat, k_hat
  do i=1, quad_order*ne
    write (out_unit, *) J_list(i)%vector
  end do
close(out_unit)
print *, 'Done writing current densities, J, to file.'

end program
